//##################################################
//# $Id: Main.cpp 19 2005-03-01 15:15:06Z peciva $

/*****************************************************************************\
 *
 * Main.cpp
 *
 * Test application for Infinite Terrain Engine
 *
 * Authors: Martin Havlek (xhavli15 AT stud.fit.vutbr.cz)
 *          PCJohn (peciva AT fit.vutbr.cz)
 * Contributors: 
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code.
 *
\*****************************************************************************/


#ifdef _WIN32
#  include <Inventor/Win/SoWin.h>
#  include <Inventor/Win/viewers/SoWinExaminerViewer.h>
#  pragma comment(linker, "/entry:\"mainCRTStartup\"")
#  pragma comment(linker, "/subsystem:\"console\"")
#else
#  include <Inventor/Qt/SoQt.h>
#  include <Inventor/Qt/SoQtRenderArea.h>
#endif

#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoDirectionalLight.h>
#include <Inventor/nodes/SoPerspectiveCamera.h>

#include <cstdio>  // printf
#include <cstdlib> // putenv, exit
#include <cmath>   // floor
#include <ctime>   // time

#include "TeEngine.h"

#define APP_TITLE "Static Terrain Demo"
#define APP_LAST_UPDATE "2005-05-08"
#define VIEWER_BGCOLOR SbColor(0.33f, 0.74f, 0.93f)

// ??? TEMPORARY:
// use "app-object" instead and store all important values in it ???
static TeEngine *engine = NULL;


//----------------------------------------------------------------------------
// main
int main(int argc, char *argv[])
{
  // SoWin & Coin3D libraries init
  #ifdef _WIN32
    HWND window = SoWin::init(argv[0]);
  #else
    QWidget *window = SoQt::init(argv[0]);
  #endif
  if (window == NULL) exit(1);

  // random num. generator init
  srand((unsigned int)time(NULL));

  // scene root
  SoSeparator *root = new SoSeparator;
  root->ref();

  // scene separator
  SoSeparator *sceneRoot = new SoSeparator;
  root->addChild(sceneRoot);

  // sunlight
  SoDirectionalLight *sunLight = new SoDirectionalLight;
  sunLight->direction.setValue(0.75f, 0.5f, -1.0f);
  sunLight->intensity.setValue(1.0);
  root->addChild(sunLight);

  // terrain engine
  engine = new TeEngine(SbVec2s(65, 65), SbVec2f(64.0f, 64.0f));

  TePatch *tmp = NULL;
  int x, y = 0;

  // 3x3 main patches
  for (y=0; y<3; y++)
    for (x=0; x<3; x++) {
      tmp = engine->getPatch(SbVec2f(float(x)*64.0f, float(y)*64.0f));
      tmp->prepareBuildMaps();
      tmp->buildHMapFromBuildMaps();
      root->addChild( tmp->getPatchGraph() );
    }

  // patch in the middle provides all possible seams
  tmp = engine->getPatch(SbVec2f(1*64.0f, 1*64.0f));
  for (x=TePatch::SOUTH_WEST; x<=TePatch::NORTH; x++)
    root->addChild( tmp->getSeamGraph(TePatch::Direction(x)) );

  // patches in the corners provide the rest of seams
  tmp = engine->getPatch(SbVec2f(0*64.0f, 0*64.0f));
  root->addChild( tmp->getSeamGraph(TePatch::EAST) );
  root->addChild( tmp->getSeamGraph(TePatch::NORTH) );
  tmp = engine->getPatch(SbVec2f(2*64.0f, 0*64.0f));
  root->addChild( tmp->getSeamGraph(TePatch::WEST) );
  root->addChild( tmp->getSeamGraph(TePatch::NORTH) );
  tmp = engine->getPatch(SbVec2f(0*64.0f, 2*64.0f));
  root->addChild( tmp->getSeamGraph(TePatch::EAST) );
  root->addChild( tmp->getSeamGraph(TePatch::SOUTH) );
  tmp = engine->getPatch(SbVec2f(2*64.0f, 2*64.0f));
  root->addChild( tmp->getSeamGraph(TePatch::WEST) );
  root->addChild( tmp->getSeamGraph(TePatch::SOUTH) );

  // display FPS in the viewer
  putenv("COIN_SHOW_FPS_COUNTER=1");

  // viewer
  #ifdef _WIN32
    SoWinExaminerViewer *viewer = new SoWinExaminerViewer(window);
  #else
    SoQtExaminerViewer *viewer = new SoQtExaminerViewer(window);
  #endif
  viewer->setBackgroundColor( VIEWER_BGCOLOR );
  viewer->setSceneGraph(root);
  viewer->setTitle( APP_TITLE );
  viewer->setHeadlight(false);
  viewer->show();

  // console echo
  printf("%s (%s)\n\nRunning...\n\n", APP_TITLE, APP_LAST_UPDATE);

  // show window and run mainLoop
  #ifdef _WIN32
    SoWin::show(window);
    SoWin::mainLoop();
  #else
    SoQt::show(window);
    SoQt::mainLoop();
  #endif

  // free memory
  delete viewer;
  root->unref();

  // console echo
  printf("\nDone...\n");

  return 0;
}
