#ifndef TE_ENGINE_EXPLORER_H
#define TE_ENGINE_EXPLORER_H

/*****************************************************************************\
 *
 * TeEngineExplorer.h
 *
 * Engine object that displays stats while flying above surface
 *
 * Author: Martin Havlek (xhavli15 AT stud.fit.vutbr.cz)
 * Contributors:
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code.
 *
\*****************************************************************************/

/**
 * TeEngineExplorer class header file.
 *
 * \file TeEngineExplorer.h
 * \author Martin Havlek (martyn AT seznam.cz)
 */

#include <Inventor/nodes/SoText2.h>

#include "objects/TeObject.h"

//class TeEngine;

/**
 * Simple flyer, its HUD is used to display information about the engine.
 *
 * This class is here to provide example of creating new objects that can
 * be inserted into the engine. Handling of events and HUD-related things
 * are implemented, so user can understand the meaning of them quickly.
 *
 * The HUD of this "flyer" is used to display some useful information about
 * the running engine.
 */
class TeEngineExplorer : public TeObject {

  friend class TeEngine;

private:

  /** Node that contains info about object position. \sa updateHUD() */
  SoText2 *posText;
  /** Node that contains info about object speed. \sa updateHUD() */
  SoText2 *speedText;
  /** Node that contains info about current FPS value. \sa updateHUD() */
  SoText2 *fpsText;
  /** Node that contains info about number of tasks waiting in the queue. \sa updateHUD() */
  SoText2 *tasksInQueueText;
  /** Node that contains info about total number of generated patches. \sa updateHUD() */
  SoText2 *patchNumText;
  /** Node that contains info about current patches-tree depth. \sa updateHUD() */
  SoText2 *rootLevelText;
  /** Default camera distance. */
  SbVec3f defaultCamDistance;

protected:

  virtual void createHUD();
  virtual void updateHUD();

public:

  TeEngineExplorer();

  virtual void handleEvent(const SoEvent *event);

};


#endif /* TE_ENGINE_EXPLORER_H */
